import 'dart:io';
import 'package:flutter/material.dart';

class WifiRetriever extends StatefulWidget {
  final double size;
  final Color circleColor;
  final Color wifiColor;
  final double circleStroke;
  final Color ondoneCircle;
  final Color ondoneWifi;
  final String messageText;
  final Color messageTextColor;
  final double messageTextSize;
  final String contentText;
  final Color contentTextColor;
  final double contentTextSize;
  final String reconnectingText;
  final String connectedText;
  final Duration durationToPop;
  WifiRetriever(
      {Key key,
        @required this.circleStroke,
        @required this.size,
        this.durationToPop,
        this.messageText,
        this.contentText,
        this.reconnectingText,
        this.connectedText,
        this.messageTextColor,
        this.messageTextSize,
        this.contentTextColor,
        this.contentTextSize,
        this.ondoneCircle,
        this.ondoneWifi,
        this.circleColor,
        this.wifiColor,})
      : super(key: key);

  _WifiRetrieverState createState() => _WifiRetrieverState();
}

class _WifiRetrieverState extends State<WifiRetriever>
    with TickerProviderStateMixin {
  AnimationController controller;

  AnimationController controllerDialog;
  Animation<double> scaleAnimation;

  double firstsize = 0.0;
  double secondsize = 0.0;
  double thirdsize = 0.0;
  bool netchecker = false;
  String reconnected;

  @override
  void initState() {
    super.initState();
    reconnected = widget.reconnectingText ?? 'Reconnecting...';
    /// ***
    /// *** THIS IS FOR MODAL ANIMATION AND DESIGNS ONLY
    controllerDialog =
        AnimationController(vsync: this, duration: Duration(milliseconds: 350));
    scaleAnimation =
        CurvedAnimation(parent: controllerDialog, curve: Curves.easeInOutCubic);

    controllerDialog.addListener(() {
      setState(() {});
    });

    /// ***
    /// *** THIS IS FOR WIFI ANIMATION AND DESIGNS ONLY
    // ** set FIRST controller for animation
    controller =
        AnimationController(vsync: this, duration: Duration(seconds: 2));

    // ** add listener
    controller.addListener(() {
      if (controller.value > 0.1 && controller.value < .4) {
        firstsize = 1.0;
      }

      if (controller.value > 0.4 && controller.value < .6) {
        secondsize = 1.0;
      }

      if (controller.value > 0.6 && controller.value < .9) {
        thirdsize = 1.0;
      }

      if (controller.value > .9) {
        if (netchecker) {
          firstsize = 1.0;
          secondsize = 1.0;
          thirdsize = 1.0;
          controller.forward();
        } else {
          firstsize = 0.0;
          secondsize = 0.0;
          thirdsize = 0.0;
        }
      }
      // ** rebuild widget
      setState(() {});
    });
    // ** WIFI REPEATING DESIGN ANIMATION
    controller.repeat();
    // **  CHEKCER FOR WIFI CONNECTION
    alwaysCheck();
    // ** FOR DIALOG OPENING ANIMATION
    controllerDialog.forward();
  }

  @override
  void dispose() {
    controller.dispose();
    controllerDialog.dispose();
    super.dispose();
  }

  alwaysCheck() async {
    await InternetAddress.lookup('google.com').then((data) async {
      await Future.delayed(Duration(milliseconds: 1000));
      reconnected = widget.connectedText ?? 'Connected.';
      netchecker = true;
      setState(() {});
      await Future.delayed(widget.durationToPop ?? Duration(milliseconds: 2000));
      Navigator.pop(context);
    }).catchError((onError) async {
      await Future.delayed(Duration(milliseconds: 1000));
      alwaysCheck();
    });
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: <Widget>[
        WillPopScope(
            onWillPop: () async => false,
            child: Align(
              alignment: Alignment.topCenter,
              child: Material(
                color: Colors.transparent,
                child: ScaleTransition(
                  scale: scaleAnimation,
                  child: Container(
                    padding: EdgeInsets.all(10),
                    decoration: BoxDecoration(
                        color: Colors.grey[200],
                        borderRadius: BorderRadius.only(
                            bottomLeft: Radius.circular(5),
                            bottomRight: Radius.circular(5))),
                    width: MediaQuery.of(context).size.width / 100 * 100,
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: <Widget>[
                        Expanded(
                          child: Container(
                              alignment: Alignment.bottomLeft,
                              child: Column(
                                mainAxisAlignment: MainAxisAlignment.start,
                                children: <Widget>[
                                  Padding(
                                    padding: EdgeInsets.only(bottom: 7),
                                    child: Text(
                                      '${widget.messageText ?? 'Whoops!'}',
                                      style: TextStyle(
                                          fontSize:
                                          widget.messageTextSize ?? MediaQuery.of(context).size.width /
                                              100 *
                                              7,
                                          color: widget.messageTextColor ?? Colors.red),
                                    ),
                                  ),
                                  Padding(
                                    padding: const EdgeInsets.only(bottom: 10),
                                    child: Text(
                                      '${widget.contentText ?? 'No Internet connection found.\nCheck your connection.'}',
                                      textAlign: TextAlign.center,
                                      style: TextStyle(
                                          fontSize:
                                          widget.contentTextSize ?? MediaQuery.of(context).size.width /
                                              100 *
                                              3,
                                          color: widget.contentTextColor ?? Colors.grey[600]),
                                    ),
                                  ),
                                ],
                              )
                          ),
                        ),

                        Expanded(
                          child: Container(
                              alignment: Alignment.centerRight,
                              child: Column(
                                mainAxisAlignment: MainAxisAlignment.end,
                                crossAxisAlignment: CrossAxisAlignment.end,
                                children: <Widget>[
                                  Padding(
                                    padding: EdgeInsets.only(right: 25),
                                    child: Stack(
                                      children: <Widget>[
                                        // ** CIRCLE
                                        AnimatedOpacity(
                                          duration: Duration(milliseconds: 500),
                                          opacity:
                                          netchecker ? controller.value : 1.0,
                                          child: RepaintBoundary(
                                            child: CustomPaint(
                                              child: Container(
                                                width: widget.size,
                                                height: widget.size,
                                              ),
                                              painter: CircleDesign(
                                                ctx: context,
                                                size: widget.size,
                                                isdone: netchecker,
                                                color: netchecker
                                                    ? widget.ondoneCircle ??
                                                    Colors.grey
                                                    : widget.circleColor ??
                                                    Colors.grey,
                                                circlestroke: widget.circleStroke,
                                              ),
                                            ),
                                          ),
                                        ),
                                        // ** WIFI one
                                        AnimatedOpacity(
                                            duration: Duration(milliseconds: 500),
                                            opacity: firstsize,
                                            child: RepaintBoundary(
                                              child: CustomPaint(
                                                child: Container(
                                                  width: widget.size,
                                                  height: widget.size,
                                                ),
                                                painter: WifiOne(
                                                  ctx: context,
                                                  size: widget.size,
                                                  color: netchecker
                                                      ? widget.ondoneWifi ??
                                                      Colors.white
                                                      : widget.wifiColor ??
                                                      Colors.grey,
                                                ),
                                              ),
                                            )),
                                        // ** WIFI two
                                        AnimatedOpacity(
                                          duration: Duration(milliseconds: 500),
                                          opacity: secondsize,
                                          child: RepaintBoundary(
                                            child: CustomPaint(
                                              child: Container(
                                                width: widget.size,
                                                height: widget.size,
                                              ),
                                              painter: WifiTwo(
                                                ctx: context,
                                                size: widget.size,
                                                color: netchecker
                                                    ? widget.ondoneWifi ??
                                                    Colors.white
                                                    : widget.wifiColor ?? Colors.grey,
                                              ),
                                            ),
                                          ),
                                        ),
                                        // ** WIFI three
                                        AnimatedOpacity(
                                          duration: Duration(milliseconds: 500),
                                          opacity: thirdsize,
                                          child: RepaintBoundary(
                                            child: CustomPaint(
                                              child: Container(
                                                width: widget.size,
                                                height: widget.size,
                                              ),
                                              painter: WifiThree(
                                                ctx: context,
                                                size: widget.size,
                                                color: netchecker
                                                    ? widget.ondoneWifi ??
                                                    Colors.white
                                                    : widget.wifiColor ?? Colors.grey,
                                              ),
                                            ),
                                          ),
                                        ),
                                      ],
                                    ),
                                  ),
                                  FlatButton(
                                    splashColor: Colors.transparent,
                                    highlightColor: Colors.transparent,
                                    padding: EdgeInsets.all(0),
                                    materialTapTargetSize:
                                    MaterialTapTargetSize.shrinkWrap,
                                    child: Text(
                                      '$reconnected',
                                      style: TextStyle(
                                          fontSize:
                                          MediaQuery.of(context).size.width /
                                              100 *
                                              3.5,
                                          color: Colors.grey[600]
                                      ),
                                    ),
                                    onPressed: () {},
                                  ),
                                ],
                              )
                          ),
                        ),


                      ],
                    ),
                  ),
                ),
              ),
            ))
      ],
    );
  }
}

// ** wifi Three design
class WifiThree extends CustomPainter {
  final ctx;
  final double size;
  final Color color;
  WifiThree({this.ctx, this.color, this.size});

  @override
  void paint(Canvas canvas, Size size) {
    wifiDesignThree(canvas);
  }

  wifiDesignThree(Canvas canvas) {
    Paint paint = Paint()
      ..color = color
      ..style = PaintingStyle.fill
      ..strokeWidth = 2.0;

    Path path = Path();
    path.moveTo((size / 4.2), size / 2.3);
    path.quadraticBezierTo(size / 2, size / 3.5, size / 1.28, size / 2.25);
    path.lineTo((size / 1.15), (size / 2.9));
    path.quadraticBezierTo(size / 2, size / 7.5, size / 6, size / 3);
    path.lineTo((size / 4.2), (size / 2.3));
    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }
}

// ** wifi two design
class WifiTwo extends CustomPainter {
  final ctx;
  final double size;
  final Color color;
  WifiTwo({this.ctx, this.color, this.size});

  @override
  void paint(Canvas canvas, Size size) {
    wifiDesignTwo(canvas);
  }

  wifiDesignTwo(Canvas canvas) {
    Paint paint = Paint()
      ..color = color
      ..style = PaintingStyle.fill
      ..strokeWidth = 2.0;

    Path path = Path();
    path.moveTo((size / 2.8), size / 1.7);
    path.quadraticBezierTo(size / 2, size / 2, size / 1.52, size / 1.68);
    path.lineTo((size / 1.4), (size / 1.9));
    path.quadraticBezierTo(size / 2, size / 2.51, size / 3.3, size / 1.93);
    path.lineTo((size / 2.8), (size / 1.7));
    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }
}

// ** wifi first design
class WifiOne extends CustomPainter {
  final ctx;
  final double size;
  final Color color;
  WifiOne({this.ctx, this.color, this.size});

  @override
  void paint(Canvas canvas, Size size) {
    wifiDesignOne(canvas);
  }

  wifiDesignOne(Canvas canvas) {
    Paint paint = Paint()
      ..color = color
      ..style = PaintingStyle.fill
      ..strokeWidth = 2.0;

    Path path = Path();
    path.moveTo((size / 2), size / 1.3);
    path.lineTo((size / 2.4), (size / 1.5));
    path.quadraticBezierTo(size / 2, size / 1.7, size / 1.7, size / 1.495);
    path.lineTo((size / 2), (size / 1.3));
    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }
}

// ** CIRCLE
class CircleDesign extends CustomPainter {
  final ctx;
  final double size;
  final double circlestroke;
  final Color color;
  final bool isdone;
  CircleDesign(
      {this.ctx, this.isdone, this.circlestroke, this.color, this.size});

  @override
  void paint(Canvas canvas, Size size) {
    circle(canvas);
  }

  //done
  circle(Canvas canvas) {
    var paint = Paint();
    paint.color = color;
    paint.style = isdone == null
        ? PaintingStyle.stroke
        : isdone ? PaintingStyle.fill : PaintingStyle.stroke;
    paint.strokeWidth = circlestroke;

    canvas.drawCircle(Offset(size / 2, size / 2), (size / 2) - 1, paint);

    paint.strokeWidth = 5;
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }
}
